unit LMGlue;
(*
# Copyright Department of Computer Science
# University of Western Australia
# Created : Quinn
# Station : Eriodon
# Date : Thursday, 9 January 1992
*)
interface

{$ifc undefined THINK_Pascal}
	uses
		Types,
		Files,
{$elsec}
	uses
{$endc}
		AppleTalk,
		
		LMTypes;

(*
	Notes
	-----	
	All these calls (except GetLMRefNum) are based on PBControl calls to the
	 LabMaster driver.  As such any of the calls can return any of the error messages 
	 traditionally associated with PBControl calls.  Specifically if you pass in a bad grefnum 
	 the Device Manager is going to return an error not documented here.  You have been warned.
	Also if you call one of the optional calls without checking for the appropriate functionality
	 via LMGestalt then LabMaster will return controlErr (ie -17).
	The other primary cause of failure is if LabMaster runs out of memory.  In that case it
	 should return memFullErr.
	This interface file is defined in such a way that (without setting compiler flags) you can
	 only call routines that are always defined.  The interface is structured into parts
	 separated by *****.  Each part is marked either 'always present' in which case the calls
	 defined within are present on every LabMaster driver or with the name of a compiler
	 variable and the name of a flag.  To use those calls you must set the compiler variable
	 when you import the interface *and* test for the presence of the call via a
	 LMGestalt(lm_functionality, result) call.
*)

(* ***** General Interface - always present ***** *)

	const
		bad_grefnum = maxint;

	function  GetLMRefNum(var grefnum : integer) : OSErr;
(* any err associated with OpenDriver
   if GetLMRefNum returns an error code it will also set grefnum to bad_grefnum.
*)

	function  LMGestalt(grefnum : integer; selector : OSType; var result : univ longint) : OSErr;
(* undefinedSelectorErr *)


(* ***** Control Interface - always present ***** *)

	function  RegisterCapabilities(grefnum : integer; addr : addrIdent;
																	caps : capHandle; refcon : univ longint;
																	a5world : univ Ptr) : OSErr;
(* dupAddrErr, tooManyAddrErr
		any error returned by module's CapStartup code
*)
	function  DeregisterCapabilities(grefnum : integer; addr : addrIdent) : OSErr;
(* addrNotFoundErr, labmasterErr
		any error returned by module's CapShutdown
*)

	function	SetCapTime(grefnum : integer; addr : addrIdent; deltat : longint) : OSErr;
(* addrNotFoundErr *)


(* ***** Packets - always present ***** *)

	function  SendMessage(grefnum : integer; msg : messagePtr) : OSErr;
(* badMesgErr (abs(mesgsize) > max_mesg_size or (source.addr should be registered or null_addr)),
		machNotFound, labmasterErr, tooManyMesgErr, addrNotFound*, capNotFound*
	Errors marked with a * will most probably only be returned if the message was sent to
	 the local machine.  This is only a hint and may change.  But the rule regarding
	 capMessageCompleted *always* holds.
*)
(* source.addr  -> you're addr (null_addr if you don't have one)
	 source.mach <-  your true machine
	 dest.addr   <-> where to send message (null_addr is OK)
	                 returns the true machine number of the dest
	 dest.mach    -> machine to send message to (no_machine is OK)
	 mesgID			  -> the cap that receives the message
	 seqnumber    -> setup buy LabMaster (or left if not auto_gen_seqnumber)
	 mesgsize     -> size of following data
*)
(* if a message is targetted at null_addr or no_machine then SendMessage is almost guaranteed
		to return noErr.
	 if SendMessage returns noErr and source.addr is not null_addr then
	 		LabMaster will send a capMessageCompleted (or if available a capMessageReturn)
			with err field set appropriately
	if a message is targetted at broadcast_addr then capMessageCompleted/capMessageReturn
		of noErr means that the target machine looked for addresses to deliver the message.
*)


(* ***** Streams - test gof_streams, define gofStreams ***** *)

(* Note
	 ----
	 These calls are based on the underlying ADSP protocol and as such each of the calls
	  can return errors traditionally returned by ADSP (errState for example).
*)

{$ifc not undefined gofStreams}

	function  StreamOpenPassive(grefnum : integer; addr : addrIdent; var port : streamAddr; var strm : streamRef) : OSErr;
(* tooManyStreamsErr *)

	function  StreamOpenActive(grefnum : integer; addr : addrIdent; port : streamAddr; var strm : streamRef) : OSErr;
(* tooManyStreamsErr, badPortErr *)

	function  StreamClose(grefnum : integer; strm : streamRef) : OSErr;
(* badStreamErr *)

	function  StreamRead(grefnum : integer; strm : streamRef; buf : Ptr; size : longint) : OSErr;
(* badStreamErr *)

	function  StreamWrite(grefnum : integer; strm : streamRef; buf : Ptr; size : longint) : OSErr;
(* badStreamErr *)

	function  StreamStatus(grefnum : integer; strm : streamRef; var inf : streamInfo) : OSErr;
(* badStreamErr *)

{$endc}


(* ***** Module Interface - test gof_hot_calls, define gofHotCalls ***** *)

{$ifc not undefined gofHotCalls}

	function  HotInstallModule(grefnum : integer; fss : FSSpec; var themod : modIdent) : OSErr;
(* labmasterErr, dupModuleErr (module already installed),tooManyModulesErr, badModuleErr (module structure not correct)
		any error returned by the module's entrypoint
*)
  function  HotRemoveModule(grefnum : integer; themod : modIdent) : OSErr;
(* moduleNotFoundErr (themod is not a module), labmasterErr 
		any error returned by the module's exitpoint
*)

{$endc}


(* ***** Information Interface - test gof_driver_info, define gofDriverInfo ***** *)

{$ifc not undefined gofDriverInfo}

	function  GetModuleCount(grefnum : integer; var nummods : integer) : OSErr;
	function  GetIndModule(grefnum : integer; index : integer; var amod : modIdent) : OSErr;
(* indexErr *)

	function  GetModuleInfo(grefnum : integer; themod : modIdent; version : integer; var inf : modInfo) : OSErr;
(* moduleNotFoundErr *)

	function  GetAddressCount(grefnum : integer; var numaddr : integer) : OSErr;
	function  GetIndAddress(grefnum : integer; index : integer; var addr : addrIdent) : OSErr;
(* indexErr *)
	function  GetAddressInfo(grefnum : integer; addr : addrIdent; version : integer; var inf : addrInfo) : OSErr;
(* addrNotFoundErr *)
	
	function  GetCapabilityCount(grefnum : integer; addr : addrIdent; var numcaps : integer) : OSErr;
(* addrNotFoundErr *)
	function  GetIndCapability(grefnum : integer; addr : addrIdent; index : integer; var cap : capIdent) : OSErr;
(* indexErr, addrNotFoundErr *)
	function  GetCapabilityInfo(grefnum : integer; addr : addrIdent; cap : capIdent; version : integer; var inf : capInfo) : OSErr;
(* addrNotFoundErr, capNotFoundErr *)
	
{$endc}


(* ***** Client Info Interface - test gof_client_info, defined gofClientInfo ***** *)

{$ifc not undefined gofClientInfo}

(* The Client and Server Information interfaces are now obsolete and subsumed by
		the Machine Information Interface.  DO NOT USE THEM!
*)

	function  GetClientCount(grefnum : integer; var numclients : integer) : OSErr;
	function  GetIndClient(grefnum : integer; index : integer; var mach : machIdent) : OSErr;
(* indexErr
	Your own machine always appears in the client list.  Thus if client calls are available then
	 GetIndClient(gref, 1, mach) will always return your machine number in mach.
*)
	function  GetClientInfo(grefnum : integer; mach : machIdent; var inf : machInfo) : OSErr;
(* machineNotFoundErr *)

{$endc}


(* ***** Server Info Interface - test gof_server_info, defined gofServerInfo ***** *)

{$ifc not undefined gofServerInfo}

(* The Client and Server Information interfaces are now obsolete and subsumed by
		the Machine Information Interface.  DO NOT USE THEM!
*)

	function  GetServerCount(grefnum : integer; var numserv : integer) : OSErr;
	function  GetIndServer(grefnum : integer; index : integer; var mach : machIdent) : OSErr;
(* indexErr *)
	function  GetServerInfo(grefnum : integer; mach : machIdent; var inf : machInfo) : OSErr;
(* machineNotFoundErr *)

{$endc}

(* ***** Machine Info Interface - test gof_machine_info, defined gofMachineInfo ***** *)

{$ifc not undefined gofMachineInfo}

	function  GetMachineCount(grefnum : integer; var nummach : integer) : OSErr;
	function  GetIndMachine(grefnum : integer; index : integer; var mach : machIdent) : OSErr;
(* indexErr *)
	function  GetMachineInfo(grefnum : integer; mach : machIdent; version : integer; var inf : machineInfo) : OSErr;
(* machineNotFoundErr *)

{$endc}

(* ***** Secure - test gof_secure, define gofSecure ***** *)

{$ifc not undefined gofSecure}
	function SendSecureMessage(grefnum : integer; msg : messagePtr) : OSErr;
		(* see SendMessage above *)

	function SetKey(grefnum : integer; newkey : cipherBlock; keylife : longint) : OSErr;
		(* key life in ticks, infinite_key_life to make key not timeout *)
		(* when a key times out it is set to no_key *)

{$endc}

implementation

	function  GetLMRefNum(var grefnum : integer) : OSErr;
		external;
	function  LMGestalt(grefnum : integer; selector : OSType; var result : univ longint) : OSErr;
		external;
	function	SetCapTime(grefnum : integer; addr : addrIdent; deltat : longint) : OSErr;
		external;
	function  RegisterCapabilities(grefnum : integer; addr : addrIdent;
																	caps : capHandle; refcon : univ longint;
																	a5world : univ Ptr) : OSErr;
		external;
	function  DeregisterCapabilities(grefnum : integer; addr : addrIdent) : OSErr;
		external;
	function  SendMessage(grefnum : integer; msg : messagePtr) : OSErr;
		external;
{$ifc not undefined gofStreams}
	function  StreamOpenPassive(grefnum : integer; addr : addrIdent; var port : streamAddr; var strm : streamRef) : OSErr;
		external;
	function  StreamOpenActive(grefnum : integer; addr : addrIdent; port : streamAddr; var strm : streamRef) : OSErr;
		external;
	function  StreamClose(grefnum : integer; strm : streamRef) : OSErr;
		external;
	function  StreamRead(grefnum : integer; strm : streamRef; buf : Ptr; size : longint) : OSErr;
		external;
	function  StreamWrite(grefnum : integer; strm : streamRef; buf : Ptr; size : longint) : OSErr;
		external;
	function  StreamStatus(grefnum : integer; strm : streamRef; var inf : streamInfo) : OSErr;
		external;
{$endc}
{$ifc not undefined gofHotCalls}
	function  HotInstallModule(grefnum : integer; fss : FSSpec; var themod : modIdent) : OSErr;
		external;
  function  HotRemoveModule(grefnum : integer; themod : modIdent) : OSErr;
		external;
{$endc}
{$ifc not undefined gofDriverInfo}
	function  GetModuleCount(grefnum : integer; var nummods : integer) : OSErr; (* new *)
		external;
	function  GetIndModule(grefnum : integer; index : integer; var amod : modIdent) : OSErr;
		external;
	function  GetModuleInfo(grefnum : integer; themod : modIdent; version : integer; var inf : modInfo) : OSErr;
		external;
	function  GetAddressCount(grefnum : integer; var numaddr : integer) : OSErr; (* new *)
		external;
	function  GetIndAddress(grefnum : integer; index : integer; var addr : addrIdent) : OSErr;
		external;
	function  GetAddressInfo(grefnum : integer; addr : addrIdent; version : integer; var inf : addrInfo) : OSErr;
		external;
	function  GetCapabilityCount(grefnum : integer; addr : addrIdent; var numcaps : integer) : OSErr; (* new *)
		external;
	function  GetIndCapability(grefnum : integer; addr : addrIdent; index : integer; var cap : capIdent) : OSErr;
		external;
	function  GetCapabilityInfo(grefnum : integer; addr : addrIdent; cap : capIdent; version : integer; var inf : capInfo) : OSErr;
		external;
{$endc}
{$ifc not undefined gofClientInfo}
	function  GetClientCount(grefnum : integer; var numclients : integer) : OSErr; (* new *)
		external;
	function  GetIndClient(grefnum : integer; index : integer; var mach : machIdent) : OSErr;
		external;
	function  GetClientInfo(grefnum : integer; mach : machIdent; var inf : machInfo) : OSErr;
		external;
{$endc}
{$ifc not undefined gofServerInfo}
	function  GetServerCount(grefnum : integer; var numserv : integer) : OSErr; (* new *)
		external;
	function  GetIndServer(grefnum : integer; index : integer; var mach : machIdent) : OSErr;
		external;
	function  GetServerInfo(grefnum : integer; mach : machIdent; var inf : machInfo) : OSErr;
		external;
{$endc}

{$ifc not undefined gofMachineInfo}
	function  GetMachineCount(grefnum : integer; var nummach : integer) : OSErr;
		external;
	function  GetIndMachine(grefnum : integer; index : integer; var mach : machIdent) : OSErr;
		external;
	function  GetMachineInfo(grefnum : integer; mach : machIdent; version : integer; var inf : machineInfo) : OSErr;
		external;
{$endc}

{$ifc not undefined gofSecure}
	function SendSecureMessage(grefnum : integer; msg : messagePtr) : OSErr;
		external;
	function SetKey(grefnum : integer; newkey : longint; keylife : longint) : OSErr;
		external;
{$endc}

end. (* LMGlue *)
