unit LMModuleSubs;
(*
# Copyright Department of Computer Science
# University of Western Australia
# Created : Quinn
# Station : Eriodon
# Date : Wednesday, 8 January 1992
*)
interface

{$ifc undefined THINK_Pascal}
	uses
		Types,
		Files,
{$elsec}
	uses
{$endc}
		AppleTalk,
		
		LMTypes;

(* Note
   ----
	 Use these routines in preference to parsing the message yourself.  These routines
	  are very small (being implemented in assembler), much smaller than you could
		easily write.  They also give you some degree of data abstraction.
*)

(* ***** Capability Code ***** *)

(* Note
   ----
	 To use this code you should basically create a capHandle (size 0) then
	  clear it.  Add each capability in turn.  Then call ValidCapabilities to see
		whether all went well.
*)

	procedure ClearCapabilities(h : CapHandle);
	procedure AddCapability(h : CapHandle; capID : capIdent; capProcAddr : ProcPtr);
	function  ValidCapabilities(h : CapHandle) : OSErr;
(* memFullErr *)


(* ***** Silly Utilities ***** *)

	function  BuildMesgID(id : mesgIdent) : capIdent;
		inline
			$3F5F, $0002,			(* move.w (sp)+, 2(sp) *)
			$3EBC, $6D65;			(* move.w #'me', (sp)  *)
(* Note this must change if you change the definition of capMessageID *)

	function  BuildSecureMesgID(id : mesgIdent) : capIdent;
		inline
			$3F5F, $0002,			(* move.w (sp)+, 2(sp) *)
			$3EBC, $736D;			(* move.w #'sm', (sp)  *)
(* Note this must change if you change the definition of capSecureMessageID *)

	function  BuildBroadcastID(id : mesgIdent) : capIdent;
		inline
			$3F5F, $0002,			(* move.w (sp)+, 2(sp) *)
			$3EBC, $6272;			(* move.w #'br', (sp)  *)
(* Note this must change if you change the definition of capBroadcastID *)

	function  BuildSecureBroadcastID(id : mesgIdent) : capIdent;
		inline
			$3F5F, $0002,			(* move.w (sp)+, 2(sp) *)
			$3EBC, $7362;			(* move.w #'sb', (sp)  *)
(* Note this must change if you change the definition of capSecureBroadcastID *)

	function  BuildMesgRetID(id : mesgIdent) : capIdent;
		inline
			$3F5F, $0002,			(* move.w (sp)+, 2(sp) *)
			$3EBC, $6D72;			(* move.w #'mr', (sp)  *)
(* Note this must change if you change the definition of capMessageReturnID *)

	function  StringToMesgIdent(s : Str255) : mesgIdent;
	function  MesgIdentToString(mid : mesgIdent) : Str255;

	function  SampleA5 : Ptr;
		inline
			$2E8D;						(* move.l	a5,(sp) ; store A5 into function result *)

(* ***** Routines for building messages ***** *)

(* Notes
   -----
	 The idea here is that you allocate an entire messageRecord and then rewrite it.
	  If your replying to a message then RewriteReply will automatically fill out
		the source, dest and mesgID fields correctly (and copy the seqnumber from the
		message).  If you're sending a new message then RewriteMessage is appropriate
		and you will have to fill out the entire header (except mesgsize and seqnumber) 
		yourself.
	 Then you should call AddXxxxToMessage to build the appropriate message data.
	 Then you should call ValidMessage to check that the message didn't get too big.
	  This isn't yet implemented so you can't call it.  Be careful with those bytes,
		Eugene.
	 Then call SendMessage.
	 Easy stuff!
*)

	procedure RewriteMessage(mesg : messagePtr);
	procedure RewriteReply(mesg : messagePtr; reply : messagePtr);
(* Calling RewriteReply more than once is most probably not clever.  Don't set
    mesg and reply to point to the same message.  It would be bad.
*)

	procedure AddBytesToMessage(mesg : messagePtr; fromwhere : Ptr; n : integer);
	procedure AddByteToMessage(mesg : messagePtr; b : byte);
	procedure AddStringToMessage(mesg : messagePtr; s : Str255);
	procedure AddStr32ToMessage(mesg : messagePtr; s : Str32);
	procedure AddBooleanToMessage(mesg : messagePtr; b : boolean);
	procedure AddIntegerToMessage(mesg : messagePtr; int : integer);
	procedure AddLongintToMessage(mesg : messagePtr; long : longint);
	procedure AddOSTypeToMessage(mesg : messagePtr; ot : OSType);
	procedure AddCharToMessage(mesg : messagePtr; ch : char);

(* function ValidMessage(mesg : messagePtr) : OSErr; NYI *)


(* ***** Routines for parsing message ***** *)

(* Notes
   -----
	 The idea here is that you call ResetMessage which returns a magic cookie (ndx).
	  ndx is actually the index into the data field of the messageRecord but that
		is a) not important and b) not guaranteed.
   Then call GetXxxxFromMessage passing in (and getting back) the same ndx.  If
	 	there is not enough bytes in the message then you'll get a tooFewBytesErr.
	 The message having too many bytes is not detected (and most probably shouldn't be).
*)

	procedure ResetMessage(mesg : messagePtr; var ndx : integer);

	function  GetBytesFromMessage(mesg : messagePtr; var ndx : integer; towhere : Ptr; n : integer) : OSErr;
(* tooFewBytesErr *)
	function  GetByteFromMessage(mesg : messagePtr; var ndx : integer; var b : byte) : OSErr;
(* tooFewBytesErr *)
	function  GetStringFromMessage(mesg : messagePtr; var ndx : integer; var s : Str255) : OSErr;
(* tooFewBytesErr *)
	function  GetStr32FromMessage(mesg : messagePtr; var ndx : integer; var s : Str32) : OSErr;
(* tooFewBytesErr, stringTooLongErr *)
	function  GetBooleanFromMessage(mesg : messagePtr; var ndx : integer; var b : boolean) : OSErr;
(* tooFewBytesErr *)
	function  GetIntegerFromMessage(mesg : messagePtr; var ndx : integer; var int : integer) : OSErr;
(* tooFewBytesErr *)
	function  GetLongintFromMessage(mesg : messagePtr; var ndx : integer; var long : longint) : OSErr;
(* tooFewBytesErr *)
	function  GetOSTypeFromMessage(mesg : messagePtr; var ndx : integer; var ot : OSType) : OSErr;
(* tooFewBytesErr *)
	function  GetCharFromMessage(mesg : messagePtr; var ndx : integer; var ch : char) : OSErr;
(* tooFewBytesErr *)

(* function GotAllBytes(mesg : messagePtr; ndx : integer) : OSErr; NYI *)

implementation

	procedure ClearCapabilities(h : CapHandle);
		external;
	procedure AddCapability(h : CapHandle; capID : capIdent; capProcAddr : ProcPtr);
		external;
	function  ValidCapabilities(h : CapHandle) : OSErr;
		external;

	function  StringToMesgIdent(s : Str255) : mesgIdent;
		external;
	function  MesgIdentToString(mid : mesgIdent) : Str255;
		external;
		
	procedure RewriteMessage(mesg : messagePtr);
		external;
	procedure RewriteReply(mesg : messagePtr; reply : messagePtr);
		external;

	procedure AddBytesToMessage(mesg : messagePtr; fromwhere : Ptr; n : integer);
		external;
	procedure AddByteToMessage(mesg : messagePtr; b : byte);
		external;
	procedure AddStringToMessage(mesg : messagePtr; s : Str255);
		external;
	procedure AddStr32ToMessage(mesg : messagePtr; s : Str32);
		external;
	procedure AddBooleanToMessage(mesg : messagePtr; b : boolean);
		external;
	procedure AddIntegerToMessage(mesg : messagePtr; int : integer);
		external;
	procedure AddLongintToMessage(mesg : messagePtr; long : longint);
		external;
	procedure AddOSTypeToMessage(mesg : messagePtr; ot : OSType);
		external;
	procedure AddCharToMessage(mesg : messagePtr; ch : char);
		external;

	procedure ResetMessage(mesg : messagePtr; var ndx : integer);
		external;

	function  GetBytesFromMessage(mesg : messagePtr; var ndx : integer; towhere : Ptr; n : integer) : OSErr;
		external;
	function  GetByteFromMessage(mesg : messagePtr; var ndx : integer; var b : byte) : OSErr;
		external;
	function  GetStringFromMessage(mesg : messagePtr; var ndx : integer; var s : Str255) : OSErr;
		external;
	function  GetStr32FromMessage(mesg : messagePtr; var ndx : integer; var s : Str32) : OSErr;
		external;
	function  GetBooleanFromMessage(mesg : messagePtr; var ndx : integer; var b : boolean) : OSErr;
		external;
	function  GetIntegerFromMessage(mesg : messagePtr; var ndx : integer; var int : integer) : OSErr;
		external;
	function  GetLongintFromMessage(mesg : messagePtr; var ndx : integer; var long : longint) : OSErr;
		external;
	function  GetOSTypeFromMessage(mesg : messagePtr; var ndx : integer; var ot : OSType) : OSErr;
		external;
	function  GetCharFromMessage(mesg : messagePtr; var ndx : integer; var ch : char) : OSErr;
		external;

end. (* LMModuleSubs *)
