/*
# Copyright Department of Computer Science
# University of Western Australia
# Created : Quinn
# Station : Eriodon
# Date : Tuesday, 13 October 1992
# From : LMTypes.p Rev 11

	The canonical LabMaster interface is defined in Pascal.  These headers have not
	been thoroughly tested.  If there is a conflict between these headers and the
	Pascal interfaces, the Pascal should take precedence.
*/

/* //////////////////////////////////////////////////////////////////////////////// */

#ifndef __LMTYPES__
#define __LMTYPES__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __OSUTILS__
#include <OSUtils.h>
#endif

#ifndef __APPLETALK__
#include <AppleTalk.h>
#endif

/* //////////////////////////////////////////////////////////////////////////////// */

#define record_size_limit 32000

/* integerHandle and longIntHandle were not defined because C people aren't
    heavily into naming types.  This policy is also applied to other definitions
    later in this file (marked with >>>)
*/

typedef long machIdent;
/* >>> machIdentPtr */

enum {
	/* constants for machIdent */
	my_machine = 1,
	no_machine = 0,
	server_machine = -1
};

typedef unsigned short mesgIdent;

typedef OSType addrIdent;
/* >>> addrIdentPtr */
/* constants for addrIdent */
#define null_addr 'null'
#define broadcast_addr 'brod'

typedef OSType modIdent;
/* >>> modIdentPtr */
/* constants for modIdent */
#define null_mod 'null'

typedef OSType capIdent;
/* >>> capIdentPtr */

struct mesgAddr {
	machIdent mach;
	addrIdent addr;
};
typedef struct mesgAddr mesgAddr;
/* >>> mesgAddrPtr */

struct capRecord {
	capIdent cap;
	ProcPtr capaddr;
};
typedef struct capRecord capRecord;

#define cap_header_size sizeof(short)
#define cap_record_size sizeof(capRecord)

enum {
		max_caps = 4000
};

struct capData {
	short count;
	capRecord caps[max_caps];
};
typedef struct capData capData;
typedef capData *capPtr, **capHandle;

enum {
	latest_env_record_version = 1
};

struct envRecordVersion0 {
	long version;
	short grefnum;
	modIdent myModIdent;
	FSSpec myfile;
};
typedef struct envRecordVersion0 envRecordVersion0;

struct envRecord {
	long version;
	short grefnum;
	modIdent myModIdent;
	FSSpec myfile;
	Boolean init_time;
};
typedef struct envRecord envRecord;
typedef envRecord *envPtr, **envHandle;

struct modOptions {
	modIdent modid;
	long extraheapsize;
};
typedef struct modOptions modOptions;
typedef modOptions *modOptionsPtr, **modOptionsHandle;

enum {
	max_mesg_size = 512,
	max_mesg_size_1 = max_mesg_size - 1,
	mesg_size_mask = 0x000003FFL,
	mesg_flag_mask = 0x0000FC00L,
	mesg_readonly_mask = 0x00008000L,
	mesg_secure_mask = 0x00004000L,
	auto_gen_seqnumber = -1L,
	min_max_message_lifetime = 2*60*60
};

struct messageHeader {
	mesgAddr source;
	mesgAddr dest;
	mesgIdent mesgID;
	long seqnumber;
	short mesgsize;
};
typedef struct messageHeader messageHeader;
struct messageRecord {
	messageHeader header;
	unsigned char data[max_mesg_size];
};
typedef struct messageRecord messageRecord;
typedef messageRecord *messagePtr;

typedef long streamRef;
/* >>> streamRefPtr */
typedef long streamAddr[4];
/* >>> streamAddrPtr */

enum {
	null_stream_ref = 0
};

struct cipherBlock {
	long hi;
	long lo;
};
typedef struct cipherBlock cipherBlock;

enum {
	/* error numbers */ 
	labMasterErr = -25000,
	dupModuleErr = -25001,
	tooManyModulesErr = -25002,
	badModuleErr = -25003,
	moduleNotFoundErr = -25004,
	indexErr = -25005,
	addrNotFoundErr = -25006,
	capNotFoundErr = -25007,
	machineNotFoundErr = -25008,
	dupAddrErr = -25009,
	tooManyAddrErr = -25010,
	badMesgErr = -25011,
	tooManyMesgErr = -25012,
	undefinedSelectorErr = -25013,
	labMasterTooSmallErr = -25014,
	tooFewBytesErr = -25015,
	badStreamErr = -25016,
	autoIndexErr = -25017,
	noConfigErr = -25018,
	tooManyStreamsErr = -25019,
	badPortErr = -25020,
	dupCapErr = -25021,
	timeoutErr = -25022,
	lostMesgErr = -25023,
	stringTooLongErr = -25024,
	tooManyCapErr = -25025
};

#define lm_functionality 'func'
enum {
	/* bit numbers and masks for lm_functionality selector response */
	gof_hot_calls = 0,						gof_hot_calls_mask = 1,
	gof_driver_info = 1,					gof_driver_info_mask = 2,
	gof_client_info = 2,					gof_client_info_mask = 4,				/* obsolete */
	gof_server_info = 3,					gof_server_info_mask = 8,				/* obsolete */
	gof_streams = 4,							gof_streams_mask = 16,
	gof_secure = 5,								gof_secure_mask = 32,
	gof_broadcast = 7,						gof_broadcast_mask = 128,
	gof_machine_info = 8,					gof_machine_info_mask = 256
};

#define lm_machine_attr 'mach'
enum {
	/* bit numbers and masks for lm_machine_attr selector response */
			lma_server = 0,								lma_server_mask = 1,
			lma_remote_info = 1,					lma_remote_info_mask = 2,
			lma_secure_links = 2,					lma_secure_links_mask = 4,
			lma_init_time = 3,						lma_init_time_mask = 8,
			lma_secure_active = 4,				lma_secure_active_mask = 16
};

#define lm_type 'type'
enum {
	/* values for lm_type selector response */
			gof_type_server = 1,
			gof_type_client = 0
};

#define lm_client_count 'clcn'

#define lm_server_count 'svcn'

#define lm_max_streams 'maxs'

#define lm_cur_streams 'curs'

#define lm_version 'lmvr'

#define lm_machine_ident 'midt'

/* //////////////////////////////////////////////////////////////////////////////// */

#define capMessageCompletedID 'cmpm'
/* pascal void CapMessageCompleted(long refcon, messagePtr mesg, OSErr err); */

#define capMessageReturnID 'mrxx'
/* pascal void CapMessageReturn(long refcon, messagePtr mesg, OSErr err); */

#define capStartupID 'init'
/* pascal OSErr CapStartup(long refcon); */

#define capShutdownID 'shut'
/* pascal OSErr CapShutdown(long refcon); */


#ifdef gofClientInfo

/* The Client and Server Information interfaces are now obsolete and subsumed by
		the Machine Information Interface.  DO NOT USE THEM!
*/

#define capFoundClientID 'fncl'
/* pascal void CapFoundClient(long refcon, machIdent mach); */

#define capLostClientID 'lscl'
/* pascal void CapLostClient(long refcon, machIdent mach); */

#endif

#ifdef gofServerInfo

/* The Client and Server Information interfaces are now obsolete and subsumed by
		the Machine Information Interface.  DO NOT USE THEM!
*/

#define capFoundServerID 'fnsv'
/* pascal void CapFoundServer(long refcon, machIdent mach); */

#define capLostServerID 'lssv'
/* pascal void CapLostServer(long refcon, machIdent mach); */

#endif

#define capFoundMachineID 'fnma'
/* pascal void CapFoundMachine(long refcon, machIdent mach); */

#define capLostMachineID 'lsma'
/* pascal void CapLostMachine(long refcon, machIdent mach); */


#define capTimeID 'time'
/* pascal void CapTime(long refcon, long *deltat); */
enum {
	no_more_time = -1
};

#define capMessageID 'mexx'
/* pascal void CapMessage(long refcon, messagePtr mesg); */
/* pascal OSErr CapMessage(long refcon, messagePtr mesg); */


#ifdef gofStreams

#define capStreamEstablishedID 'stes'
/* pascal void CapStreamEstablished(long refcon, streamRef strm); */

#define capStreamClosedID 'stcl'
/* pascal void CapStreamClosed(long refcon, streamRef strm); */

#define capCharsFreeID 'stcf'
/* pascal void CapCharsFree(long refcon, streamRef strm, const streamInfo *info); */

#define capCharsAvailID 'stca'
/* pascal void CapCharsAvail(long refcon, streamRef strm, const streamInfo *info); */

#endif

#define capSecureMessageID 'smxx'
/* pascal void CapSecureMessage(long refcon, messagePtr mesg); */
/* pascal OSErr CapSecureMessage(long refcon, messagePtr mesg); */

#ifdef gofSecure

#define capRequestKeyID 'rqky'
/* pascal void CapRequestKey(long refcon); */
enum {
	no_key = 0,										// must be in hi and lo cipherBlock
	infinite_key_life = -1,
	min_key_life = 5 * 60 * 60
};
#endif

#ifdef gofBroadcast

#define capBroadcastID 'brxx'
/* pascal void CapBroadcast(long refcon, messagePtr mesg); */

#define capSecureBrodcastID 'sbxx'
/* pascal void CapSecureBroadcast(long refcon, messagePtr mesg); */

#endif

/* //////////////////////////////////////////////////////////////////////////////// */

enum {
	latest_machine_info_version = 0,
	latest_mod_info_version = 0,
	latest_addr_info_version = 0,
	latest_cap_info_version = 0
};

struct machineInfo {
	short version;
	machIdent mach;
	Str32 name;
	char fill1;
	Str32 zone;
	char fill2;
	Str32 clique;
	char fill3;
	AddrBlock adr;
	long attr;
};
typedef struct machineInfo machineInfo;
/* >>> machineInfoPtr */

struct machInfo {
	Str32 name;
	char fill1;
	Str32 zone;
	char fill2;
	Str32 clique;
	char fill3;
	AddrBlock adr;
};
typedef struct machInfo machInfo;
/* >>> machInfoPtr */

struct modInfo {
	short version;
	modIdent modid;
	FSSpec modfile;
	Handle modglobals;
	ProcPtr modcode;
};
typedef struct modInfo modInfo;
/* >>> modInfoPtr */

struct addrInfo {
	short version;
	addrIdent addr;
	long refcon;
	Ptr a5world;
	long lastdelta;
	long time;
};
typedef struct addrInfo addrInfo;
/* >>> addrInfoPtr */

struct capInfo {
	short version;
	addrIdent addr;
	capIdent cap;
	ProcPtr capaddr;
};
typedef struct capInfo capInfo;
/* >>> capInfoPtr */

struct streamInfo {
	Boolean connected;
	short sendpending;
	short sendfree;
	short receivepending;
	short receivefree;
};
typedef struct streamInfo streamInfo;
/* >>> streamInfoPtr */

#endif
