unit LMTypes;
(*
# Copyright Department of Computer Science
# University of Western Australia
# Created : Quinn
# Station : Eriodon
# Date : Wednesday, 8 January 1992
*)
interface

(* Conditional Compilation Notes
   -----------------------------
	 The following compiler variables are valid throughout the LabMaster source code:
		o goParanoia if true then we include a large amount of defensive code
		o goDevelopment if true then we're building an experimental LabMaster
	 The following compiler variables should mirror the bits in the lm_functionality
	  selector of the LMGestalt call:
		o gofHotCalls if true then we're allowed HotInstall/Remove calls
		o gofDriverInfo if true then the driver info calls are available
		o gofClientInfo if true then the client info calls are available (obsolete)
		o gofServerInfo if true then the server info calls are available (obsolete)
		o gofStreams if true then the stream calls are available
		o gofSecure if true then the secure message calls are available
		o gofBroadcast if true then broadcast message are available
		o gofMachineInfo if true then the machine info calls are available
*)


(*  ->  means field must be setup
   <-  means field is returned
	 <-> means field must be setup and is returned
	  #  means field should not be setup
	  ?  means field value is undefined
		
	This syntax applies to both calls to LabMaster and to the way LabMaster calls a capability.
	eg -> on SendMessage means you should init the field
	   -> on CapError means LabMaster should init the field
	Jager's mnemonic is callee on the left, caller on the right.  Remember that Jager
	 can't tell left from right.  So in fact it is...
	 
	"Caller on the left, callee on the right"
	
	Are you confused yet?
*)

{$ifc undefined THINK_Pascal}
	uses
		Types,
		Files,
{$elsec}
	uses
{$endc}
		AppleTalk;
	
	const
		record_size_limit = 32000;							{stop pascal from using long indexing}

	type
		integerHandle = ^integerPtr;
		longIntHandle = ^longIntPtr;
		
		machIdent = longint;
		machIdentPtr = longIntPtr;

	const
		my_machine = 1;
		no_machine = 0;
		server_machine = -1;

	type
		mesgIdent = packed array [1..2] of char;
		addrIdent = OSType;
		addrIdentPtr = OSTypePtr;
	const
		null_addr = 'null';
		broadcast_addr = 'brod';
	type
		modIdent = OSType;
		modIdentPtr = OSTypePtr;
	const
		null_mod = 'null';
	type
		capIdent = OSType;
		capIdentPtr = OSTypePtr;
		
		mesgAddr =
			record
				mach : machIdent;
				addr : addrIdent;
			end;
		mesgAddrPtr = ^mesgAddr;
		
		capRecord =
			record
				cap : capIdent;
				capaddr : ProcPtr;
			end;
	const
		cap_header_data_size = sizeof(integer);		{old, do not use}
		cap_header_size = sizeof(integer);
		cap_record_size = sizeof(capRecord);
		max_caps = 4000;
	type
		capData = 
			record
				count : integer;
				caps : array [1..max_caps] of capRecord;
			end;
		capPtr = ^capData;
		capHandle = ^capPtr;
		
	const
		latest_env_record_version = 1;
	type
		envRecordVersion0 = 
			record
				version : longint;
				grefnum : integer;
				myModIdent : modIdent;
				myfile : FSSpec;
			end;
		envRecord = 
			record
				version : longint;
				grefnum : integer;
				myModIdent : modIdent;
				myfile : FSSpec;
				init_time : boolean;
			end;
		envPtr = ^envRecord;
		envHandle = ^envPtr;
		
		modOptions =
			record
				modid : modIdent;
				extraheapsize : longint;
			end;
		modOptionsPtr = ^modOptions;
		modOptionsHandle = ^modOptionsPtr;
	
	const
		max_mesg_size = 512;
		max_mesg_size_1 = max_mesg_size - 1;		(* evil Think!!! *)
		mesg_size_mask = $000003FF;							(* masks to extract relevant *)
		mesg_flag_mask = $0000FC00;							(* portions of the mesgsize field *)
		mesg_readonly_mask = $00008000;					(* top bit means that the mesg is read only *)
		mesg_secure_mask = $00004000;						(* next bit means that the mesg is secure *)
		auto_gen_seqnumber = -1;
		min_max_message_lifetime = 2*60*60;	(* messages can last at least this number of ticks *)
	type
		messageNdx = 0..max_mesg_size_1;
		messageHeader =
			record
				source : mesgAddr;
				dest : mesgAddr;
				mesgID : mesgIdent;
				seqnumber : longint;
				mesgsize : integer;
			end;
		messageRecord =
			record
				header : messageHeader;
				data : array [messageNdx] of signedByte;
			end;
		messagePtr = ^messageRecord;
	
	type
		streamRef = longint;
		streamRefPtr = ^streamRef;
		streamAddr = array [1..4] of longint;
		streamAddrPtr = ^streamAddr;
	const
		null_stream_ref = 0;
		
	type
		cipherBlock = record								(* a 64bit block for signatures, keys *)
										hi,lo : longint;		(* and 2way random numbers *)
									end;
	const
		labMasterErr = -25000;						(* driver internal error *)
		dupModuleErr = -25001;
		tooManyModulesErr = -25002;
		badModuleErr = -25003;
		moduleNotFoundErr = -25004;
		indexErr = -25005;
		addrNotFoundErr = -25006;
		capNotFoundErr = -25007;
		machineNotFoundErr = -25008;
		dupAddrErr = -25009;
		tooManyAddrErr = -25010;
		badMesgErr = -25011;							(* invalid format message *)
		tooManyMesgErr = -25012;					(* buffer overrun *)
		undefinedSelectorErr = -25013;
		labMasterTooSmallErr = -25014;
		tooFewBytesErr = -25015;
		badStreamErr = -25016;
		autoIndexErr = -25017;
		noConfigErr = -25018;
		tooManyStreamsErr = -25019;
		badPortErr = -25020;							(* illegal stream port *)
		dupCapErr = -25021;								(* only one cap of this type allowed *)
		timeoutErr = -25022;							(* message undelivered *)
		lostMesgErr = -25023;							(* message may or may not have been delivered *)
		stringTooLongErr = -25024;				(* string is too long to be got from message *)
		tooManyCapErr = -25025;						(* too many caps in capHandle *)
		
(* Available Selectors *)

  const
		lm_functionality = 'func';			(* result is bit field *)
			gof_hot_calls = 0;						gof_hot_calls_mask = 1;
			gof_driver_info = 1;					gof_driver_info_mask = 2;
			gof_client_info = 2;					gof_client_info_mask = 4;		(* obsolete *)
			gof_server_info = 3;					gof_server_info_mask = 8;		(* obsolete *)
			gof_streams = 4;							gof_streams_mask = 16;
			gof_secure = 5;								gof_secure_mask = 32;
			
			gof_broadcast = 7;						gof_broadcast_mask = 128;
			gof_machine_info = 8;					gof_machine_info_mask = 256;
			
		lm_machine_attr = 'mach';				(* result is bit field *)
			lma_server = 0;								lma_server_mask = 1;
			lma_remote_info = 1;					lma_remote_info_mask = 2;
			lma_secure_links = 2;					lma_secure_links_mask = 4;
			lma_init_time = 3;						lma_init_time_mask = 8;			{dynamic, not valid remotly}
			lma_secure_active = 4;				lma_secure_active_mask = 16;{dynamic, not valid remotly}
					
		lm_type = 'type';								(* result is enum *)
			gof_type_server = 1;
			gof_type_client = 0;

		lm_client_count = 'clcn';				(* result is count *)
		
		lm_server_count = 'svcn';				(* result is count *)
		
		lm_max_streams	=	'maxs';				(* result is count *)
		
		lm_cur_streams	= 'curs';				(* result is count *)
		
		lm_version			= 'lmvr';				(* result is version longint *)
		
		lm_machine_ident = 'midt';			(* result is machIdent *)
		

(* A5 World *)
(*
		Caps are called with A5 set to the value passed into a5world when they are registered.
		 If a5world was nil then the cap gets LabMaster's A5.  Don't touch!!!
		A5 does not need to be preserved by a cap call since LabMaster restores its own after
		 the call.
*)

	const
		capMessageCompletedID = 'cmpm';
(* procedure CapMessageCompleted(refcon : longint; mesg : messagePtr; err : OSErr) *)
(* err  -> addrNotFound, capNotFound, labmasterErr, tooManyMesgErr, machineNotFoundErr,
		lostMesgErr,timeoutErr, or as returned by capMessage/capSecureMessage
		other fields are as set as returned by SendMessage (except -ve mesgSize)
		
		err=noErr  message sent successfully
		err<>noErr message not sent/error return from message cap
*)

	const
(* IMPORTANT:  if you change this you must change the inline BuildMesgReturnID in LMModuleSubs *)
		capMessageReturnID = 'mrxx';			(* xx is any two chars *)

(* procedure CapMessageReturn(refcon : longint; mesg : messagePtr; err : OSErr) *)
(* err  -> addrNotFound, capNotFound, labmasterErr, tooManyMesgErr, machineNotFoundErr,
		lostMesgErr,timeoutErr, or as returned by capMessage/capSecureMessage
		other fields are as set as returned by SendMessage (except -ve mesgSize)
		
		err=noErr  message sent successfully
		err<>noErr message not sent/error return from message cap
*)

		capStartupID = 'init';
(* function CapStartup(refcon : longint) : OSErr; *)
(*   globals are refcon from RegisterCapabilities *)
(*   result passed back to caller of RegisterCapabilities *)

		capShutdownID = 'shut';
(* function CapShutDown(refcon : longint) : OSErr; *)
(*   globals are never to be seen again *)
(*   result passed back to caller of DeregisterCapabilities *)

{$ifc not undefined gofClientInfo}

(* The Client and Server Information interfaces are now obsolete and subsumed by
		the Machine Information Interface.  DO NOT USE THEM!
*)

	 capFoundClientID = 'fncl';
(* procedure CapFoundClient(refcon : longint; mach : machIdent); *)
(* mach -> ID of client found *)

	 capLostClientID = 'lscl';
(* procedure CapLostClient(refcon : longint; mach : machIdent); *)
(* mach -> ID of client lost *)

{$endc}

{$ifc not undefined gofServerInfo}

(* The Client and Server Information interfaces are now obsolete and subsumed by
		the Machine Information Interface.  DO NOT USE THEM!
*)

	 capFoundServerID = 'fnsv';
(* procedure CapFoundServer(refcon : longint; mach : machIdent); *)
(* mach -> ID of server found *)

	 capLostServerID = 'lssv';
(* procedure CapLostServer(refcon : longint; mach : machIdent); *)
(* mach -> ID of server lost *)

{$endc}

	 capFoundMachineID = 'fnma';
(* procedure CapFoundMachine(refcon : longint; mach : machIdent); *)
(* mach -> ID of machine found *)

	 capLostMachineID = 'lsma';
(* procedure CapLostMachine(refcon : longint; mach : machIdent); *)
(* mach -> ID of machine lost *)

		capTimeID = 'time';
(* procedure CapTime(refcon : longint; var deltat : longint); *)
(*   deltat <- time in ticks before CapTime called again *)
		no_more_time = -1;

(* IMPORTANT:  if you change this you must change the inline BuildMesgID in LMModuleSubs *)
		capMessageID = 'mexx';			(* xx is any two chars *)

(* procedure CapMessage(refcon : longint; mesg : messagePtr); *)
(* or *)
(* function CapMessage(refcon : longint; mesg : messagePtr) : OSErr; *)

(* source.addr -> addr of program that sent message (possible null_addr)
	 source.mach -> machine number of source
	 dest.addr   -> your address
	 dest.mach   -> your machine
	 mesgID			 -> should match one of the caps that you registered
	 seqnumber   -> seqnumber as set by sender/LabMaster
	 mesgsize    -> size of following data
*)

(* if a function is used then the error returned will be returned to the sender *)
(* if a procedure is used then noErr will be returned to indicate delivery *)

(* ----- Streams ----- *)

{$ifc not undefined gofStreams}

	 capStreamEstablishedID = 'stes';
(* procedure CapStreamEstablished(refcon : longint; strm : streamRef); *)

	 capStreamClosedID = 'stcl';
(* procedure CapStreamClosed(refcon : longint; strm : streamRef); *)

	 capStreamCharsFreeID = 'stcf';
(* procedure CapStreamCharsFree(refcon : longint; strm : streamRef; info : streamInfo); *)

	 capStreamCharsAvailID = 'stca';
(* procedure CapStreamCharsAvail(refcon : longint; strm : streamRef; info : streamInfo); *)

{$endc}

(* ----- Secure ----- *)

(* IMPORTANT:  if you change this you must change the inline BuildSecureMesgID in LMModuleSubs *)
		capSecureMessageID = 'smxx';			(* xx is any two chars *)

(* procedure CapSecureMessage(refcon : longint; mesg : messagePtr); *)
(* or *)
(* function CapSecureMessage(refcon : longint; mesg : messagePtr) : OSErr; *)

(* source.addr -> addr of program that sent message (possible null_addr)
	 source.mach -> machine number of source
	 dest.addr   -> your address
	 dest.mach   -> your machine
	 mesgID			 -> should match one of the caps that you registered
	 seqnumber   -> seqnumber as set by sender/LabMaster
	 mesgsize    -> size of following data
*)

{$ifc not undefined gofSecure}

(* if a function is used then the error returned will be returned to the sender *)
(* if a procedure is used then noErr will be returned to indicate delivery *)

(* message delivery *)
(* SendMessage will be delivered to the capMessage if present otherwise errors *)
(* SendSecureMessage will be delivered to the capSecureMessage if present *)
(* otherwise to the capMessage if present otherwise errors *)

		capRequestKeyID = 'rqky';
(* procedure CapRequestKey(refcon : longint); *)
(* need a new key to send a secure message *)
		no_key = 0; {must be in hi and lo of param}
		infinite_key_life = -1;
		min_key_life = 5*60*60;		{minimum key life is 5 minutes}

{$endc}

(* ----- Broadcast ----- *)

{$ifc not undefined gofBroadcast}

(* IMPORTANT:  if you change this you must change the inline BuildBroadID in LMModuleSubs *)
		capBroadcastID = 'brxx';			(* xx is any two chars *)

(* procedure CapBroadcast(refcon : longint; mesg : messagePtr); *)

(* source.addr -> addr of program that sent message (possible null_addr)
	 source.mach -> machine number of source
	 dest.addr   -> your address
	 dest.mach   -> your machine
	 mesgID			 -> should match one of the caps that you registered
	 seqnumber   -> seqnumber as set by sender/LabMaster
	 mesgsize    -> size of following data
*)

(* IMPORTANT:  if you change this you must change the inline BuildSecureBroadID in LMModuleSubs *)
		capSecureBroadcastID = 'sbxx';			(* xx is any two chars *)

(* procedure CapSecureBroadcast(refcon : longint; mesg : messagePtr); *)

(* source.addr -> addr of program that sent message (possible null_addr)
	 source.mach -> machine number of source
	 dest.addr   -> your address
	 dest.mach   -> your machine
	 mesgID			 -> should match one of the caps that you registered
	 seqnumber   -> seqnumber as set by sender/LabMaster
	 mesgsize    -> size of following data
*)

{$endc}

(* ----- Types for LabMaster driver status calls ----- *)

	const
		latest_machine_info_version = 0;
		latest_mod_info_version = 0;
		latest_addr_info_version = 0;
		latest_cap_info_version = 0;

	type
		machineInfo =
			record
				version : integer;
				mach : machIdent;
				name : Str32;
				zone : Str32;
				clique : Str32;
				adr : AddrBlock;
				attr : longint;
			end;
		machineInfoPtr = ^machineInfo;
	
		machInfo =
			record
				name : Str32;
				zone : Str32;
				clique : Str32;
				adr : AddrBlock;
			end;
		machInfoPtr = ^machInfo;
		
		modInfo =
			record
				version : integer;
				modid : modIdent;
				modfile : FSSpec;
				modglobals : Handle;
				modcode : ProcPtr;
			end;
		modInfoPtr = ^modInfo;
		
		addrInfo = 
			record
				version : integer;
				addr : addrIdent;
				refcon : longint;
				a5world : Ptr;
				lastdelta : longint;			(* if lastdelta = no_more_time then capTime shan't be called anymore *)
				time : longint;
			end;
		addrInfoPtr = ^addrInfo;
		
		capInfo = 
			record
				version : integer;
				addr : addrIdent;
				cap : capIdent;
				capaddr : ProcPtr;
			end;
		capInfoPtr = ^capInfo;
		streamInfo =
			record
				connected : boolean;						(* are we connected *)
				sendpending : integer;					(* number of bytes waiting to send *)
				sendfree : integer;							(* number of bytes free in send buffer *)
				receivepending : integer;				(* number of bytes waiting *)
				receivefree : integer;					(* number of bytes free in receive buffer *)
			end;
		streamInfoPtr = ^streamInfo;
		
implementation

end. (* LMTypes *)
