unit NotifyClient;

(* Copyright Department of Computer Science *)
(* University of Western Australia *)
(* Created : Quinn *)
(* Station : Eriodon *)
(* Date : Wednesday, 16 September 1992 *)
(* This file may be freely distributed . *)

interface

{$ifc undefined THINK_Pascal}
	uses
		Types, Files, Memory, Resources, GestaltEqu, 
{$elsec}
	uses
{$endc}
		Notification, AppleTalk, 

		LMTypes, LMGlue, LMModuleSubs, 

		NotifyTypes;

	type
		noteRecord = record
				noterec: NMRec;
				mesgstr: Str255;
			end;
		noteRecordPtr = ^noteRecord;

		clientGlobals = record
				gref: integer;
				note_count: integer;
			end;
		clientGlobalsPtr = ^clientGlobals;
		clientGlobalsHandle = ^clientGlobalsPtr;

	function Main (env: envPtr; globals: clientGlobalsHandle; startup: boolean; var stayresident: boolean): OSErr;

implementation

(* ----- Notification Response Procedure ----- *)

	procedure NMResponseProc (noterecptr: noteRecordPtr);
		var
			globals: clientGlobalsHandle;
			junk: OSErr;
	begin
		globals := clientGlobalsHandle(noterecptr^.noterec.nmRefCon);
		globals^^.note_count := globals^^.note_count - 1;
		junk := NMRemove(@noterecptr^.noterec);
		DisposePtr(Ptr(noterecptr));
	end; (* NMResponseProc *)

(* ----- Messaging Capabilities ----- *)

	procedure CapNotifyMesg (globals: clientGlobalsHandle; mesg: messagePtr);
		var
			ndx: integer;
			err: OSErr;
			nrp: noteRecordPtr;
	begin
		nrp := noteRecordPtr(NewPtrSys(sizeof(noteRecord)));
		err := MemError;
		ResetMessage(mesg, ndx);
		if err = noErr then begin
			err := GetStringFromMessage(mesg, ndx, nrp^.mesgstr);
		end; (* if *)
		if err = noErr then begin
			with nrp^.noterec do begin (* safe *)
				qType := ord(nmType);
				nmMark := 0;
				nmIcon := nil;
				nmSound := nil;
				nmStr := @nrp^.mesgstr;
				nmResp := @NMResponseProc;
				nmRefCon := longint(globals);
			end; (* with *)
			err := NMInstall(@nrp^.noterec);
		end; (* if *)
		if err = noErr then begin
			globals^^.note_count := globals^^.note_count + 1;
		end
		else if nrp <> nil then begin
			DisposePtr(Ptr(nrp));
		end; (* if *)
	end; (* CapNotifyMesg *)

(* ----- Initialise/Terminate Code ----- *)

	function Main (env: envPtr; globals: clientGlobalsHandle; startup: boolean; var stayresident: boolean): OSErr;
		var
			response: longint;
			err: OSErr;
			cap: capHandle;
	begin
		if startup then begin
			err := Gestalt(gestaltNotificationMgrAttr, response);
			if (err = noErr) and not btst(response, gestaltNotificationPresent) then begin
				err := labMasterTooSmallErr;
			end; (* if *)
			if err = noErr then begin
				SetHandleSize(Handle(globals), sizeof(clientGlobals));
				err := MemError;
			end; (* if *)
			if err = noErr then begin
				HLock(Handle(globals));
				globals^^.gref := env^.grefnum;
				globals^^.note_count := 0;
				cap := capHandle(NewHandle(0));
				err := MemError;
			end; (* if *)
			if err = noErr then begin
				ClearCapabilities(cap);
				AddCapability(cap, BuildMesgID(capNotifyMesgID), @CapNotifyMesg);
				err := ValidCapabilities(cap);
				if err = noErr then begin
					err := RegisterCapabilities(env^.grefnum, notify_client_addr, cap, longint(globals), nil);
				end; (* if *)
				DisposeHandle(Handle(cap));
			end; (* if *)
			stayresident := (err = noErr);
		end
		else begin
			err := DeregisterCapabilities(env^.grefnum, notify_client_addr);
			stayresident := (globals^^.note_count > 0);
		end; (* if *)
		Main := err;
	end; (* Main *)

end. (* NotifyClient *)