/*
# Copyright Department of Computer Science
# University of Western Australia
# Created : Quinn
# Station : Eriodon
# Date : Wednesday, 14 October 1992
*/

#define gofSecure
#define gofMachineInfo

#include <Memory.h>
#include <OSUtils.h>

#include "LMTypes.h"
#include "LMGlue.h"
#include "LMModuleSubs.h"

// TimeServer relies on the AgentClient to actually set the time on the remote
//  machine.  One would normally #include the modules header file ie "AgentClient.h".
//  However AgentClient is written in Pascal and it's easier for me to just
//  replicate the (few) definitions I need rather than convert the entire
//  interface file.

#define agent_client_addr 'cagn'
#define capSetTimeMesgID 'sc'

#include "TimeServer.h"

struct clientGlobalsRecord {
	short gref;
	// keep these statistics just for fun
	long  num_machines_found;
	long  num_machines_attempted;
	long  num_settings_successful;
	long  num_settings_failed;
};
typedef struct clientGlobalsRecord clientGlobalsRecord,
	*clientGlobalsPtr,
	**clientGlobalsHandle;

pascal void CapFoundMachine(clientGlobalsHandle globals, machIdent mach);
pascal void CapSetTimeMesgReturn(clientGlobalsHandle globals, messagePtr mesg, OSErr err);
pascal OSErr main(envPtr env, clientGlobalsHandle globals, Boolean startup,
	Boolean *stayresident);

pascal OSErr main(envPtr env, clientGlobalsHandle globals, Boolean startup,
	Boolean *stayresident)
{
	OSErr err;
	capHandle cap;
	long response;
	
	if (startup) {
		err = LMGestalt(env->grefnum, lm_functionality, &response);
		if ((err == noErr) && ((response & (gof_secure + gof_machine_info_mask)) == 0)) {
			err = labMasterTooSmallErr;
		};
		if (err == noErr) {
			SetHandleSize((Handle) globals, sizeof(clientGlobalsRecord));
			err = MemError();
		};
		if (err == noErr) {
			(*globals)->gref = env->grefnum;
			(*globals)->num_machines_found = 0;
			(*globals)->num_settings_successful = 0;
			(*globals)->num_settings_failed = 0;
			cap = (capHandle) NewHandle(0);
			err = MemError();
		};
		if (err == noErr) {
			ClearCapabilities(cap);
			AddCapability(cap, BuildMesgRetID(capSetTimeMesgID), (ProcPtr) CapSetTimeMesgReturn);
			AddCapability(cap, capFoundMachineID, (ProcPtr) CapFoundMachine);
			err = ValidCapabilities(cap);
			if (err == noErr) {
			  err = RegisterCapabilities(env->grefnum, time_server_addr, cap, (long) globals, nil);
			};
			DisposeHandle((Handle) cap);
		};
		*stayresident = (err == noErr);
	} else {
		err = DeregisterCapabilities(env->grefnum, time_server_addr);
		*stayresident = false;
	};
	return(err);
}

pascal void CapFoundMachine(clientGlobalsHandle globals, machIdent mach)
{
	OSErr err;
	messageRecord mesg;
	unsigned long time;
	machineInfo minfo;
	
	(*globals)->num_machines_found += 1;
	err = GetMachineInfo((*globals)->gref, mach, latest_machine_info_version, &minfo);
	if ((err == noErr) & ((minfo.attr & lma_secure_links_mask) == 0)) {
		// only send the message to machines *without* secure links
		// otherwise it will attempt to upgrade the link which will result
		//  in ugly passord dialogs on connect on machines with secure links
		
		(*globals)->num_machines_attempted += 1;
		GetDateTime(&time);
		
		RewriteMessage(&mesg);
		mesg.header.source.addr = time_server_addr;
		mesg.header.dest.addr = agent_client_addr;
		mesg.header.dest.mach = mach;
		mesg.header.mesgID = capSetTimeMesgID;
		AddLongintToMessage(&mesg, time);
		err = SendSecureMessage((*globals)->gref, &mesg);
		if (err != noErr) {
			(*globals)->num_settings_failed += 1;
		};
	};
}

pascal void CapSetTimeMesgReturn(clientGlobalsHandle globals, messagePtr mesg, OSErr err)
{
	#pragma unused(mesg)
	
	if (err == noErr) {
		(*globals)->num_settings_successful += 1;
	} else {
		(*globals)->num_settings_failed += 1;
	};
}
