/****************************************************************************
 * GamePlay.c
 *
 *		Overall Game routines
 *
 * JAB 4/28/93 - 5/93
 ****************************************************************************/
 
#define SPRITEMAIN	1
#include "CToast.h"
#include "math.h"

GWorldPtr		gOffScreen;
short			gResFile;
PixMapHandle	gPixMap;
short			gMaxSprite,gSpriteCnt,gSparkCnt,gAsteroidCnt,gRemainingShips;
short			gMaxSaveMaps;
short			gYummyCnt;
long			gScreenRowBytes,gVideoRowBytes;
Ptr				gScreenMem,gVideoMem;
Rect			gPlayRect,gOffscreenRect;
long			gPlayWidth,gPlayHeight;
long			gGameClockTicks = 3L;		// 20 Frames per Second (ok on LC?)
long			gGameClock;
long			gLastDispTime = 0L;
short			gGameState,gGameLevel,gLastLevel,gLastRemainingShips;
short			gShipMode,gShieldPower,gLastShieldPower;
short			gScoreMultiply,gLastScoreMultiply;
long			gGameScore,gLastScore;
short			gBadGuyChance,gYummyChance;
short			oldMBarHeight;

SpriteInstance	*sTable,*gShip;
SaveMapRecord	*smTable;

void InitializeGame(WindowPtr theWin)
{
	short			i,j,n,b;
	GDHandle		gDevice;
	Ptr				sp,dp;
	Handle			h;
	OSErr			oe;
	GDHandle		curDevice;

	GammaFadeOut();
	curDevice = GetGDevice();
	gVideoRowBytes = (*(*curDevice)->gdPMap)->rowBytes & 0x3FFF;
	gVideoMem = (*(*curDevice)->gdPMap)->baseAddr;

	gOffscreenRect = theWin->portRect;
	gOffscreenRect.right += IconWidth*2;
	gOffscreenRect.bottom += IconHeight*2;
	if ((oe = NewGWorld(&gOffScreen,0,&gOffscreenRect,NULL,NULL,0)) != noErr) {
		DebugStr("\pOy!");
		return;
	}
	gPixMap = GetGWorldPixMap(gOffScreen);
	LockPixels(gPixMap);

	gScreenRowBytes = (*gPixMap)->rowBytes & 0x3FFF;
	gScreenMem = (*gPixMap)->baseAddr;

	gPlayRect = gOffscreenRect;
	gPlayRect.right -= IconWidth;
	gPlayRect.bottom -= IconHeight;

	LoadPreferences();	// Do this before loading sounds - has volume setting

	InitSounds();

	// Hide Menubar
	oldMBarHeight = MBarHeight;
	MBarHeight = 0;

	StartupScreen();
	GammaFadeIn();

	DrawStatusBar(0,100);

	GetDateTime((unsigned long *) &randSeed);

	PlaySound(S_Startup, 4);

	gPlayWidth = gPlayRect.right - gPlayRect.left;
	gPlayHeight = gPlayRect.bottom - gPlayRect.top;

	smTable = (SaveMapRecord *) NewPtrClear(sizeof(SaveMapRecord) * MaxSaveMaps);
	if (smTable == NULL)
		DebugStr("\pOut of Mem");

	sTable = (SpriteInstance *) NewPtrClear(sizeof(SpriteInstance) * MaxSprites);
	if (sTable == NULL)
		DebugStr("\pOut of Mem");

	for (i = 0; i < NbrSprites; ++i) {
		DrawStatusBar(i,NbrSprites+1);

		if (sDef[i].firstIconID) {
			// Allocate Pix Maps
			sDef[i].colorMaps = NewPtrClear(ColorMapSize * sDef[i].nbrIcons);
			if (sDef[i].colorMaps == NULL)
				DebugStr("\pOut of Mem");
			sDef[i].maskMaps = NewPtrClear(ColorMapSize *  sDef[i].nbrIcons);
			if (sDef[i].maskMaps == NULL)
				DebugStr("\pOut of Mem");
	
			// Load Icons
			for (j = 0; j < sDef[i].nbrIcons; ++j) {
	
				// Get Color Sprite Picture
				h = GetResource('icl8',sDef[i].firstIconID+j);
				BlockMove(*h,sDef[i].colorMaps + AniFrameIndex(j), ColorMapSize);
				ReleaseResource(h);
	
				// Get Mask, convert to 8 bit
				h = GetResource('ICN#',sDef[i].firstIconID+j);
				sp = *h + MaskMapSize;
				dp = sDef[i].maskMaps + AniFrameIndex(j);
				b = 0x80;
				for (n = 0; n < ColorMapSize; ++n) {
					*dp = ((*sp & b) > 0)? 0xFF : 0x00;
					++dp;
					b >>= 1;
					if (b == 0) {
						b = 0x80;
						++sp;
					}
				}
				ReleaseResource(h);
			}
		}
	}
	DrawStatusBar(NbrSprites,NbrSprites+1);
	LoadTopScores();
	DrawStatusBar(NbrSprites+1,NbrSprites+1);
	
	// Error Check!
	SetPort(theWin);
}

void CleanUp(void)
{
	EndSounds();
	ShowCursor();
	MBarHeight = oldMBarHeight;
	CloseResFile(gResFile);
}

void DrawStatusBar(short curLevel, short maxLevel)
{
	CGrafPtr		curPort;
	GDHandle		curDevice;
	Rect			r,r2;
	RGBColor		saveColor,foreColor;

	GetGWorld(&curPort,&curDevice);
	SetGWorld(gOffScreen,NULL);

	SetRect(&r,0,0,412,2);

	// Position within 512x384 Rectangle
	OffsetRect(&r,(512-412)/2,384-8);

	// Position within offscreen video area (which is scaled to monitor size)
	// But max 640 x 480
	OffsetRect(&r,(gOffScreen->portRect.right - 512)/2,
				  (gOffScreen->portRect.bottom - 384)/2);

	GetForeColor(&saveColor);
	foreColor.red = 0x0000;
	foreColor.green = 0x9999;
	foreColor.blue = 0xCCCC;
	RGBForeColor(&foreColor);
	PaintRect(&r);
	r2 = r;
	r2.right -= (r2.right-r2.left)-(((r2.right-r2.left)*curLevel)/maxLevel);
	foreColor.green = 0;
	RGBForeColor(&foreColor);
	PaintRect(&r2);
	RGBForeColor(&saveColor);

	SetGWorld(curPort,curDevice);

	MyCopyRect(&r);
}

// Display Attract Mode Screen
void BeginAttract(void)
{
	CGrafPtr	curPort;
	GDHandle	curDevice;
	static 		StringPtr promptStr1 = "\pHit P to Play";
	static 		StringPtr promptStr2 = "\p0-7 to change volume";
	static 		StringPtr promptStr3 = "\pK to redefine keys";


	GetGWorld(&curPort,&curDevice);

	SetGWorld(gOffScreen,NULL);
	PaintRect(&gOffScreen->portRect);

	DisplayPicture(BackgroundPICT,-1,-1);
	DisplayPicture(SmallLogoPICT,64,64);

	DisplayTopScores();

	TextFont(geneva);
	TextSize(12);
	TextFace(bold);
	TextMode(srcBic);

	MoveTo(320+32-StringWidth(promptStr1)/2,240);
	DrawString(promptStr1);

	MoveTo(320+32-StringWidth(promptStr2)/2,240+32);
	DrawString(promptStr2);

	MoveTo(320+32-StringWidth(promptStr3)/2,240+32+32);
	DrawString(promptStr3);

	TextMode(srcCopy);


	SetGWorld(curPort,curDevice);

	GammaFadeOut();
	MyCopyBits();
	GammaFadeIn();
}


void BeginGame(void)
{
	short	i;
	CGrafPtr		curPort;
	GDHandle		curDevice;
	short			eMask;

	register		SpriteInstance	*sp;

	gMaxSprite = 0;
	gMaxSaveMaps = 0;
	gSpriteCnt = 0;
	gSparkCnt = 0;
	gAsteroidCnt = 0;
	gRemainingShips = 3;
	gGameLevel = 0;
	gGameScore = 0L;
	gShieldPower = MaxShieldPower;

	GetGWorld(&curPort,&curDevice);

	SetGWorld(gOffScreen,NULL);

	TextFont(geneva);
	TextSize(9);
	TextFace(0);
	TextMode(notSrcCopy);
	PaintRect(&gOffScreen->portRect);

	DisplayPicture(BackgroundPICT,-1,-1);

	SetGWorld(curPort,curDevice);

	InitStatusDisplay();	// Display sprites should be at the lowest level

	// NewDebugDisplay();		// Debug Sprites Next

	InitLevel();			// Asteroids

	NewShip();				// Ship

	GammaFadeOut();
	MyCopyBits();
	GammaFadeIn();

	// Draw New Sprites
	for (i = 0,sp = sTable; i < gMaxSprite; ++i,++sp)
		(*sDef[sp->type].drawFunc)(sp);

	eMask = everyEvent;
	eMask &= ~keyUpMask;
	eMask &= ~keyDownMask;
	eMask &= ~autoKeyMask;
	SetEventMask(eMask);

	gGameState = GS_Play;
}

void EndGame(void)
{
	short			eMask;
	CGrafPtr		curPort;
	GDHandle		curDevice;
	static StringPtr promptStr = "\pGame Over";
	SpriteInstance	*sp;
	short			i;
	long			t;
	EventRecord		dmyEvent;

	i = gMaxSprite;
	sp = &sTable[i-1];
	while (i--) {
		if (sp->active)
			(*sDef[sp->type].eraseFunc)(sp);
		--sp;
	}

	GetGWorld(&curPort,&curDevice);

	SetGWorld(gOffScreen,NULL);

	TextFont(geneva);
	TextSize(12);
	TextFace(bold);
	TextMode(srcBic);
	MoveTo(320+32-StringWidth(promptStr)/2,240+32);
	DrawString(promptStr);

	SetGWorld(curPort,curDevice);
	MyCopyBits();


	eMask = everyEvent;
	eMask &= ~keyUpMask;
	SetEventMask(eMask);
	FlushEvents(everyEvent, 0);

	t = TickCount();

	IntegrateScore(gGameScore, gGameLevel);

	gGameState = GS_Attract;
	while (TickCount() - t < 180L &&
			!OSEventAvail(everyEvent, &dmyEvent));
			;
}

// Create a new batch of asteroids
void InitLevel(void)
{
	short	i,n;

	n = gGameLevel/2 + 3;
	if (n > 20)
		n = 20;

	while (n--)
		NewAsteroid((n+gGameLevel) % NbrAsteroids);

	gScoreMultiply = 1;

	gBadGuyChance = 1200 - 100*(gGameLevel/3);

	if (gBadGuyChance < 300)
		gBadGuyChance = 300;

	gYummyChance = 1000;
}

// Level was completed successfully
void EndLevel(void)
{
	++gGameLevel;
	AddScore(LevelBonusScore*gGameLevel);
	PlaySound(S_LevelCompletion, 4);
}

void MainGameLoop(WindowPtr theWin)
{
	register SpriteInstance	*sp;
	register short	i,n;
	register long	t;
	static EventRecord	dmyEvent;
	Rect	r;
#if DEBUGGING
	void	IdleTest();				// Debugging Function used to determine CPU load for profiler
#endif

#if __option(profile)			// 6/15 Optional profiling support
	_profile = 1;
#endif

	SetPort(theWin);

	do {
		t = Ticks;
	
		if (t - gLastDispTime >= gGameClockTicks) {
			//  Update the game clock
			++gGameClock;
			gLastDispTime = t;
	
			// Compact Sprite Table if necessary
			if (gMaxSprite > 20 && gMaxSprite > (gSpriteCnt<<1)) {
				for (i = n = 0; i < gMaxSprite; ++i) {
					if (sTable[i].active) {
						sTable[n] = sTable[i];
						if (sTable[n].saveMapPtr)
							sTable[n].saveMapPtr->sp = &sTable[n];
						if (gShip == &sTable[i])
							gShip = &sTable[n];
						++n;
					}
				}
				gMaxSprite = n;
				for (i = n = 0; i < gMaxSaveMaps; ++i) {
					if (smTable[i].active) {
						smTable[n] = smTable[i];
						smTable[n].sp->saveMapPtr = &smTable[n];
						++n;
					}
				}
				gMaxSaveMaps = n;
			}
	

			// Erase all Sprites from offscreen gworld
			i = gMaxSprite;
			sp = &sTable[i-1];
			while (i--) {
				if (sp->active)
					(*sDef[sp->type].eraseFunc)(sp);
				--sp;
			}
	

			// Increment play level if necessary
			if (gAsteroidCnt == 0) {
				// Next Level!!
				EndLevel();
				InitLevel();
			}

			// Check Keyboard
			CheckKeys();

			// Random Events
			if (MyRandom(gBadGuyChance) == 0) {	// 1200 Random Bad Guys
				if (MyRandom(30) == 0)			// Every 30 minutes
					n = MyRandom(7);			// Lucky 7!
				else
					n = 1;
				while (n--) {
					switch (MyRandom(5)) {
					case 0:
					case 1:
						NewSaucer();
						break;
					case 2:
					case 3:
						NewBarbell();
						break;
					case 4:
						NewCube();
						break;
						
					}
				}
			}

			// Random Yummies
			// If we add more types of yummies, add an occasional treasure trove
			if (gYummyCnt == 0 && MyRandom(gYummyChance) == 0)	// Random Yummys
				NewYummy();

			// Move Sprites
			n = gMaxSprite;
			for (i = 0,sp=sTable; i < n; ++i,++sp) {
				// Update Sprite Position
				if (sp->active)
					(*sDef[sp->type].moveFunc)(sp);
			}

			// Draw Sprites in Offscreen
			n = gMaxSprite;
			for (i = 0,sp=sTable; i < n; ++i,++sp) {
				// Render Sprite in New Position
				if (sp->active)
					(*sDef[sp->type].drawFunc)(sp);
			}
	
			// Render Sprites onto Onscreen video
			n = gMaxSprite;
			for (i = 0,sp=sTable; i < n; ++i,++sp) {
				// Render Sprite in New Position
				if (sp->update) {
					register Point	p1,p2;

					sp->update = false;

					p1 = sp->oldPos;
					p2 = sp->pos;
					if (abs(p2.h - p1.h) > 320 ||
						abs(p2.v - p1.v) > 240)
					{
						// Object has crossed boundaries, draw two separate rects
						r.left = p1.h;
						r.top =  p1.v;
						r.right = p1.h+sp->width;
						r.bottom = p1.v+sp->width;
						// Clip
						if (r.left < IconWidth)
							r.left = IconWidth;
						if (r.top < IconHeight)
							r.top = IconHeight;
						if (r.right >= gPlayRect.right)
							r.right = gPlayRect.right-1;
						if (r.bottom >= gPlayRect.bottom)
							r.bottom = gPlayRect.bottom-1;
						if (r.right > r.left && r.bottom > r.top)
							MyCopyRect(&r);
						r.left = p2.h;
						r.top =  p2.v;
						r.right = p2.h+sp->width;
						r.bottom = p2.v+sp->width;
						// Clip
						if (r.left < IconWidth)
							r.left = IconWidth;
						if (r.top < IconHeight)
							r.top = IconHeight;
						if (r.right >= gPlayRect.right)
							r.right = gPlayRect.right-1;
						if (r.bottom >= gPlayRect.bottom)
							r.bottom = gPlayRect.bottom-1;
						if (r.right > r.left && r.bottom > r.top)
							MyCopyRect(&r);
					}
					else {
						// Merge rects into composite rect
						r.left = (p1.h < p2.h)? p1.h : p2.h;
						r.top = (p1.v < p2.v)? p1.v : p2.v;
						r.right = (p1.h > p2.h)? p1.h : p2.h;
						r.right += sp->width;
						r.bottom = (p1.v > p2.v)? p1.v : p2.v;
						r.bottom += sp->width;
						// Clip
						if (r.left < IconWidth)
							r.left = IconWidth;
						if (r.top < IconHeight)
							r.top = IconHeight;
						if (r.right >= gPlayRect.right)
							r.right = gPlayRect.right-1;
						if (r.bottom >= gPlayRect.bottom)
							r.bottom = gPlayRect.bottom-1;
						if (r.right > r.left && r.bottom > r.top)
							MyCopyRect(&r);
					}
					sp->oldPos = sp->pos;
				}
			}
		}
#if DEBUGGING
		else {
			IdleTest();
		}
#endif
	} while (gGameState == GS_Play || (gGameState == GS_GameOver && gSparkCnt));
#if __option(profile)			// 6/15 Optional profiling support
	_profile = 0;
#endif
}

void InitStatusDisplay(void)
{
	register SpriteInstance *sp;
	if ((sp = NewSprite(false)) == NULL)
		return;
	sp->type = ST_StatusDisplay;
	sp->param1 = 0;
	sp->pos.h = 40;
	sp->pos.v = 64;
	sp->width = 2;
	gLastScore = -1;
	gLastRemainingShips = -1;
	gLastLevel = -1;
	gLastScoreMultiply = -1;
	gLastShieldPower = -1;
}

void StatusDraw(register SpriteInstance *sp)
{
	static char		scoreText[32];
	Rect			r;
	register short	cw;

	if (gLastScore != gGameScore) {
		gLastScore = gGameScore;
		StatusPrintf(40,44,"%-10ld",gGameScore);
	}
	if (gLastLevel != gGameLevel) {
		gLastLevel = gGameLevel;
		StatusPrintf(122,44,"Level: %d", gGameLevel);
	}
	if (gLastRemainingShips != gRemainingShips) {
		gLastRemainingShips = gRemainingShips;
		StatusPrintf(202,44,"Pots: %d", gRemainingShips);
	}
	if (gLastScoreMultiply != gScoreMultiply) {
		gLastScoreMultiply = gScoreMultiply;
		if (gScoreMultiply > 1)
			StatusPrintf(286, 44,"x%d     ", gScoreMultiply);
		else
			StatusPrintf(286, 44,"        ", gScoreMultiply);
	}
	if (gLastShieldPower != gShieldPower) {
		RGBColor	fc;
		Rect		r;
		CGrafPtr		curPort;
		GDHandle		curDevice;

		GetGWorld(&curPort,&curDevice);
		SetGWorld(gOffScreen,NULL);

		gLastShieldPower = gShieldPower;
		MoveTo(520,40);

		fc.red = 0x0000;	fc.green = fc.blue = 0x8080;
		RGBForeColor(&fc);
		Line(gShieldPower,0);

		fc.red = 0xFFFF;	fc.green = fc.blue = 0x0000;
		RGBForeColor(&fc);
		Line(MaxShieldPower-gShieldPower,0);

		fc.red = fc.green = fc.blue = 0x0000;
		RGBForeColor(&fc);
		SetRect(&r,520,40,620,41);
		MyCopyRect(&r);

		SetGWorld(curPort,curDevice);
	}
}

void AddScore(short amt)
{
	gGameScore += amt * (long) gScoreMultiply;
	if (gGameScore/50000L != gLastScore/50000) {
		++gRemainingShips;
		PlaySound(S_ExtraShip, 4);
	}
}

void PrintfXY(short x, short y, char *template, ...)
{
	CGrafPtr		curPort;
	GDHandle		curDevice;
	char 			tbuf[128];
	va_list 		args;
	Rect			r;
	register		short cw;

	va_start(args,template);
	vsprintf(tbuf,template,args);
	va_end(args);

	CtoPstr(tbuf);

	cw = StringWidth((StringPtr) tbuf);
	MoveTo(x,y);
	DrawString((StringPtr) tbuf);
}

void StatusPrintf(short x, short y, char *template, ...)
{
	CGrafPtr		curPort;
	GDHandle		curDevice;
	char 			tbuf[128];
	va_list 		args;
	Rect			r;
	register		short cw;

	va_start(args,template);
	vsprintf(tbuf,template,args);
	va_end(args);

	CtoPstr(tbuf);

	GetGWorld(&curPort,&curDevice);
	SetGWorld(gOffScreen,NULL);
	cw = StringWidth((StringPtr) tbuf);
	MoveTo(x,y);
	DrawString((StringPtr) tbuf);
	SetGWorld(curPort,curDevice);

	r.left = x;
	r.top = y - 10;
	r.right = x + cw;
	r.bottom = y;

	MyCopyRect(&r);
}


#define Debounce		10
#define UziDebounce		1

short	fireCtr=0;

void CheckKeys(void)
{
	static KeyMap	km;
	register short	flags;
	extern SpriteInstance	*gShip;

	GetKeys(km);


	// Non Ship Related Stuff
	//
	if ((((char *) km)[1] & 0x10) ||		// 0x0C Q
		(((char *) km)[6] & 0x20)) {		// 0x35 ESC
		gGameState = GS_GameOver;
	}

#if DEBUGGING
	if (((char *) km)[5] & 0x04) {			// 0x2A \		trigger random event
		switch (MyRandom(4)) {
		case 0:	NewYummy();		break;
		case 1:	NewSaucer(); 	break;
		case 2: NewBarbell();	break;
		case 3: NewCube();		break;
		}
	}
	if (((char *) km)[6] & 0x04) {			// 0x32 ~		trigger random event
		switch (MyRandom(2)) {
		case 0:	NewYummy();				break;
		case 1: NewAsteroid(ST_Jim);	break;
		}
	}
#endif

	if (((char *) km)[4] & 0x08) {			// 0x23 P
		do {
			GetKeys(km);
		} while (((char *) km)[4] & 0x08);	// Wait for Key Up

		// Print Pause Message
		StatusPrintf(352,44,"PAUSE");

		do {
			GetKeys(km);
		} while (!(((char *) km)[4] & 0x08));	// Wait for Key Down
		do {
			GetKeys(km);
		} while (((char *) km)[4] & 0x08);	// Wait for Key Up

		// Print UnPause Message
		StatusPrintf(352,44,"             ");
	}

	// Ship Related Stuff
	//

	if (!gShip)
		return;

	flags = false;

	if (((char *) km)[gPrefs.thrustKeyByte] & gPrefs.thrustKeyBit) {
		flags |= SF_Thrust;
	}

	if (((char *) km)[gPrefs.fireKeyByte] & gPrefs.fireKeyBit) {
		if (fireCtr == 0) {
			flags |= SF_Fire;
			fireCtr = ((gShipMode & SM_Uzi)? UziDebounce : Debounce);
		}
		else
			--fireCtr;
	}
	else
		fireCtr = 0;

	if (((char *) km)[gPrefs.leftKeyByte] & gPrefs.leftKeyBit) {
		flags |= SF_Left;
	}

	if (((char *) km)[gPrefs.rightKeyByte] & gPrefs.rightKeyBit) {
		flags |= SF_Right;
	}

	if (((char *) km)[gPrefs.shieldKeyByte] & gPrefs.shieldKeyBit) {
		if (gShieldPower > 0) {
			gShieldPower--;				// Use up shield Power
			flags |= SF_Shield;
		}
	}
	else
		if ((gGameClock & 0xFF) == 0)		// Increase Shield Power every 16 secs
			if (gShieldPower < MaxShieldPower)
				gShieldPower += 5;

	if (gShipMode & SM_AutoShield) {
		if (gShip && gShip->tickCtr > 40)
			gShipMode &= ~SM_AutoShield;
		else
			flags |= SF_Shield;
	}

	gShip->param1 = flags;

}

#if DEBUGGING
void IdleTest()
{
	short	x;
	x = 0;
}
#endif
