/**********************************************************************\

File:		Four-corner scroll.c

Purpose:	Graphic effect from offscreen bitmap to main window (on
			screen).  See comments below for more description.


MSG Demo -- graphic effects demonstration program
Copyright (C) 1992-3 Mark Pilgrim & Dave Blumenthal

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "msg timing.h"

#define		BoxSize		10
#define CorrectTime 4

void FourCornerScroll(GrafPtr, GrafPtr, int, int);

/* Four scrolls in one, each confined to a section of the screen which includes
   the center and one side, and all four scrolls moving towards the center.
   Kinda slow on my Mac IIci in 8-bit; fine in 4-bit or fewer. */
   
void FourCornerScroll(GrafPtr sourceGrafPtr, GrafPtr destGrafPtr, int theWindowHeight, int theWindowWidth)
{
	int			x;
	Rect		theTopRect, topdest, theBottomRect, bottomdest;
	Rect		theLeftRect, leftdest, theRightRect, rightdest;
	Rect		topscrollsource, topscrolldest, bottomscrollsource, bottomscrolldest;
	Rect		leftscrollsource, leftscrolldest, rightscrollsource, rightscrolldest;
	RgnHandle	toprgn,bottomrgn,leftrgn,rightrgn;
	int			cx,cy;
	int			HBoxSize=BoxSize*theWindowWidth/theWindowHeight;
	
	cx = theWindowWidth / 2;
	cy = theWindowHeight / 2;

	toprgn=NewRgn();
	SetEmptyRgn(toprgn);
	MoveTo(0,0);
	OpenRgn();                               /* top region */
		LineTo(theWindowWidth,0);
		LineTo(cx,cy);
		LineTo(0,0);
	CloseRgn(toprgn);
	
	bottomrgn=NewRgn();
	SetEmptyRgn(bottomrgn);
	MoveTo(0,theWindowHeight);
	OpenRgn();                               /* bottom region */
		LineTo(theWindowWidth,theWindowHeight);
		LineTo(cx,cy);
		LineTo(0,theWindowHeight);
	CloseRgn(bottomrgn);
	
	leftrgn=NewRgn();
	SetEmptyRgn(leftrgn);
	MoveTo(0,0);
	OpenRgn();                               /* left region */
		LineTo(0,theWindowHeight);
		LineTo(cx,cy);
		LineTo(0,0);
	CloseRgn(leftrgn);
	
	rightrgn=NewRgn();
	SetEmptyRgn(rightrgn);
	MoveTo(theWindowWidth,0);
	OpenRgn();                               /* right region */
		LineTo(theWindowWidth,theWindowHeight);
		LineTo(cx,cy);
		LineTo(theWindowWidth,0);
	CloseRgn(rightrgn);
	
	topscrollsource=bottomscrollsource=leftscrollsource=rightscrollsource=
		topdest=bottomdest=leftdest=rightdest=destGrafPtr->portRect;
	
	topscrollsource.bottom-=cy+BoxSize;
	topscrolldest = topscrollsource;
	OffsetRect(&topscrolldest, 0, BoxSize);        /* dest. strip for top scroll */
	
	topdest.bottom=BoxSize;                        /* dest. strip for new data on top */
	
	theTopRect.top=cy-BoxSize;                     /* source strip for new data on top */
	theTopRect.bottom=cy;
	theTopRect.left=0;
	theTopRect.right=theWindowWidth;
	
	bottomscrollsource.top+=cy+BoxSize;
	bottomscrolldest=bottomscrollsource;
	OffsetRect(&bottomscrolldest, 0, -BoxSize);
	
	bottomdest.top=bottomdest.bottom-BoxSize;
	
	theBottomRect.top=cy;
	theBottomRect.left=0;
	theBottomRect.bottom=cy+BoxSize;
	theBottomRect.right=theWindowWidth;
	
	leftscrollsource.right-=cx+HBoxSize;
	leftscrolldest=leftscrollsource;
	OffsetRect(&leftscrolldest,HBoxSize,0);
	
	leftdest.right=HBoxSize;
	
	theLeftRect.top=0;
	theLeftRect.bottom=theWindowHeight;
	theLeftRect.left=cx-HBoxSize;
	theLeftRect.right=cx;
	
	rightscrollsource.left+=cx+HBoxSize;
	rightscrolldest=rightscrollsource;
	OffsetRect(&rightscrolldest,-HBoxSize,0);
	
	rightdest.left=rightdest.right-HBoxSize;
	
	theRightRect.top=0;
	theRightRect.bottom=theWindowHeight;
	theRightRect.left=cx;
	theRightRect.right=cx+HBoxSize;
	
	/* for each section (top, bottom, left, right) we need to do the scroll part,
	which takes a section of the dest. window and moves it, then we need to copy
	in the new data from the source window */
	for(x = cy - BoxSize; x >= 0; x -= BoxSize)
	{
		StartTiming();
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&topscrollsource, &topscrolldest, 0, toprgn);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&theTopRect, &topdest, 0, toprgn);
		theTopRect.bottom-=BoxSize;
		theTopRect.top-=BoxSize;
		
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&bottomscrollsource, &bottomscrolldest, 0, bottomrgn);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&theBottomRect, &bottomdest, 0, bottomrgn);
		theBottomRect.top+=BoxSize;
		theBottomRect.bottom+=BoxSize;
		
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&leftscrollsource, &leftscrolldest, 0, leftrgn);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&theLeftRect, &leftdest, 0, leftrgn);
		theLeftRect.left-=HBoxSize;
		theLeftRect.right-=HBoxSize;
		
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&rightscrollsource, &rightscrolldest, 0, rightrgn);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&theRightRect, &rightdest, 0, rightrgn);
		theRightRect.left+=HBoxSize;
		theRightRect.right+=HBoxSize;
		
		TimeCorrection(CorrectTime);
	}
	
	theTopRect.top=theTopRect.left=0;
	theTopRect.bottom=theWindowHeight;
	theTopRect.right=theWindowWidth;
	topdest=destGrafPtr->portRect;
	CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
		&theTopRect, &topdest, 0, 0L);    /* in case we missed any bits */

	DisposeRgn(toprgn);
	DisposeRgn(bottomrgn);
	DisposeRgn(leftrgn);
	DisposeRgn(rightrgn);
}