/**********************************************************************\

File:		Halves scroll.c

Purpose:	Graphic effect from offscreen bitmap to main window (on
			screen).  See comments below for more description.


MSG Demo -- graphic effects demonstration program
Copyright (C) 1992-3 Mark Pilgrim & Dave Blumenthal

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "msg timing.h"

#define		BoxSize		10
#define CorrectTime 3

void HalvesScroll(GrafPtr, GrafPtr, int, int);

/* 2 regions, split down the middle of the screen.  Scroll the screen up in one
   region and down in the other. */
   
void HalvesScroll(GrafPtr sourceGrafPtr, GrafPtr destGrafPtr, int theWindowHeight, int theWindowWidth)
{
	int			x;
	Rect		theTopRect, topdest, theBottomRect, bottomdest;
	Rect		topscrollsource, topscrolldest, bottomscrollsource, bottomscrolldest;
	RgnHandle	toprgn,bottomrgn;
	int			cx;
	
	cx = theWindowWidth / 2;

	toprgn=NewRgn();
	SetEmptyRgn(toprgn);
	MoveTo(0,0);
	OpenRgn();
		Line(cx,0);
		LineTo(cx,theWindowHeight);
		LineTo(0,theWindowHeight);
		LineTo(0,0);
	CloseRgn(toprgn);
	
	bottomrgn=NewRgn();
	SetEmptyRgn(bottomrgn);
	MoveTo(cx,0);
	OpenRgn();
		LineTo(cx,theWindowHeight);
		LineTo(theWindowWidth,theWindowHeight);
		LineTo(theWindowWidth,0);
		LineTo(cx,0);
	CloseRgn(bottomrgn);
	
	topscrollsource=destGrafPtr->portRect;
	topscrollsource.bottom-=BoxSize;
	topscrollsource.right=cx;
	topscrolldest = topscrollsource;
	OffsetRect(&topscrolldest, 0, BoxSize);
	
	topdest = destGrafPtr->portRect;
	topdest.bottom=BoxSize;
	topdest.right=cx;
	
	theTopRect.top=theWindowHeight-BoxSize;
	theTopRect.bottom=theWindowHeight;
	theTopRect.left=0;
	theTopRect.right=cx;
	
	bottomscrollsource=destGrafPtr->portRect;
	bottomscrollsource.top+=BoxSize;
	bottomscrollsource.left=cx;
	bottomscrolldest=bottomscrollsource;
	OffsetRect(&bottomscrolldest, 0, -BoxSize);
	
	bottomdest=destGrafPtr->portRect;
	bottomdest.top=bottomdest.bottom-BoxSize;
	bottomdest.left=cx;
	
	theBottomRect.top=0;
	theBottomRect.bottom=BoxSize;
	theBottomRect.left=cx;
	theBottomRect.right=theWindowWidth;
	
	for(x = theWindowHeight - BoxSize; x >= 0; x -= BoxSize)
	{
		StartTiming();
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&topscrollsource, &topscrolldest, 0, toprgn);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&theTopRect, &topdest, 0, toprgn);
		theTopRect.bottom-=BoxSize;
		theTopRect.top-=BoxSize;
		
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&bottomscrollsource, &bottomscrolldest, 0, bottomrgn);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&theBottomRect, &bottomdest, 0, bottomrgn);
		theBottomRect.top+=BoxSize;
		theBottomRect.bottom+=BoxSize;
		
		TimeCorrection(CorrectTime);
	}
}