/**********************************************************************\

File:		Hilbert wipe.c

Purpose:	Graphic effect from offscreen bitmap to main window (on
			screen).  See comments below for more description.


MSG Demo -- graphic effects demonstration program
Copyright (C) 1992-3 Mark Pilgrim & Dave Blumenthal

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "msg timing.h"

/* This fills the screen by a Hilbert space-filling curve, which is defined by
   two mutually recursive drawing functions:
   
   X = left, Y, draw, right, X, draw, X, right, draw, Y, left
   Y = right, X, draw, left, Y, draw, Y, left, draw, X, right
   
   Start by drawing X at the highest recursion level from the bottomleft of the
   screen. (At recursion level 1, X and Y are null functions.)
*/

#define	RecursionLevel	4
#define vBlockSize 		20	/* ceiling(theWindowHeight / 2^RecursionLevel) */
#define hBlockSize 		32	/* ceiling(theWindowWidth / 2^RecursionLevel) */
#define CorrectTime 1

typedef char Hilby[11];

Hilby	HilbertPattern[]=
{
	{
		0x02,0x69,0x00,0x01,0x42,0x00,0x42,0x01,0x00,0x69,0x02
	},
	{
		0x01,0x42,0x00,0x02,0x69,0x00,0x69,0x02,0x00,0x42,0x01
	}
};

int				direction=0;

void HilbertWipe(GrafPtr, GrafPtr, int, int, int*, int*);
void HilbertWipeCall(GrafPtr, GrafPtr, int, int);

void HilbertWipe(GrafPtr sourceGrafPtr, GrafPtr destGrafPtr, int level, int whichpattern,
	int* x, int* y)
{
	int		i;
	Rect	source;
	
	for (i=0; i<11; i++)
	{
		switch (HilbertPattern[whichpattern][i])
		{
			case 0x01:     /* turn left */
				direction--;
				if (direction<0) direction=3;
				break;
			case 0x02:     /* turn right */
				direction++;
				if (direction==4) direction=0;
				break;
			case 0x00:    /* draw */
				StartTiming();
				source.top=*y-vBlockSize;
				source.bottom=*y;
				source.left=*x;
				source.right=*x+hBlockSize;
				CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
								&source, &source, 0, 0L);
				switch (direction)
				{
					case 0:
						*x+=hBlockSize;
						break;
					case 1:
						*y-=vBlockSize;
						break;
					case 2:
						*x-=hBlockSize;
						break;
					case 3:
						*y+=vBlockSize;
						break;
				}
				TimeCorrection(CorrectTime);
				break;
			case 0x42:    /* call X */
				if (level>1) HilbertWipe(sourceGrafPtr, destGrafPtr, level-1,0,x,y);
				break;
			case 0x69:    /* call Y */
				if (level>1) HilbertWipe(sourceGrafPtr, destGrafPtr, level-1,1,x,y);
				break;
		}
	}
}

void HilbertWipeCall(GrafPtr sourceGrafPtr, GrafPtr destGrafPtr, int theWindowHeight, int theWindowWidth)
{
	int			curx, cury;
	Rect		source;
	
	cury=theWindowHeight;
	curx=0;
	HilbertWipe(sourceGrafPtr,destGrafPtr,RecursionLevel,0,&curx,&cury);
	source.bottom=theWindowHeight;
	source.right=theWindowWidth;
	source.top=source.bottom-vBlockSize;
	source.left=source.right-hBlockSize;
	CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
				&source, &source, 0, 0L);  /* in case we missed any bits */

}
