/**********************************************************************\

File:		Pour scroll.c

Purpose:	Graphic effect from offscreen bitmap to main window (on
			screen).  See comments below for more description.


MSG Demo -- graphic effects demonstration program
Copyright (C) 1992-3 Mark Pilgrim & Dave Blumenthal

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "msg timing.h"

#define		ScrollSize	10		/* = how much up to scroll each strip */
#define		BoxSize		5		/* = width of each strip */
#define		NumStrips	100		/* = theWindowWidth/BoxSize */
#define CorrectTime 2

void PourScroll(GrafPtr, GrafPtr, int, int);

/* Scroll down in tiny strips, starting at the left and moving right.  Scroll
   strips 1-(N), then strips 1-(N+1), etc.  When strip 1 is done, don't do it
   anymore!  (Duh, but this was difficult to get right.)  So only scroll strips
   2-(N), and so on. */
   
void PourScroll(GrafPtr sourceGrafPtr, GrafPtr destGrafPtr, int theWindowHeight, int theWindowWidth)
{
	Rect		theRect[NumStrips], dest[NumStrips];
	Rect		scrollsource, scrolldest;
	int			i;
	int			startstrip,endstrip;
	
	scrollsource=destGrafPtr->portRect;
	scrollsource.bottom-=ScrollSize;
	scrollsource.left=0;
	scrollsource.right=BoxSize;
	scrolldest = scrollsource;
	OffsetRect(&scrolldest, 0, ScrollSize);
		
	for (i=0; i<NumStrips; i++)
	{
		dest[i] = destGrafPtr->portRect;
		dest[i].bottom=ScrollSize;
		dest[i].left=i*BoxSize;
		dest[i].right=dest[i].left+BoxSize;
		
		theRect[i].top=theWindowHeight-ScrollSize;
		theRect[i].bottom=theWindowHeight;
		theRect[i].left=i*BoxSize;
		theRect[i].right=theRect[i].left+BoxSize;
	}
	
	startstrip=0;
	endstrip=1;
	do
	{
		StartTiming();
		
		CopyBits(&(destGrafPtr->portBits), &(destGrafPtr->portBits),
				&scrollsource, &scrolldest, 0, 0L);
		
		for (i=startstrip; i<endstrip; i++)
		{
			CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
					&theRect[i], &dest[i], 0, 0L);
			theRect[i].bottom-=ScrollSize;
			theRect[i].top-=ScrollSize;
		}
		
		if (endstrip<NumStrips)
		{
			endstrip++;
			scrollsource.right+=BoxSize;
			scrolldest.right+=BoxSize;
		}
		if (theRect[startstrip].bottom==0)
		{
			startstrip++;
			scrollsource.left+=BoxSize;
			scrolldest.left+=BoxSize;
		}
		
		TimeCorrection(CorrectTime);
	}
	while (startstrip<endstrip);
}
