/**********************************************************************\

File:		Two corner.c

Purpose:	Graphic effect from offscreen bitmap to main window (on
			screen).  See comments below for more description.


MSG Demo -- graphic effects demonstration program
Copyright (C) 1992-3 Mark Pilgrim & Dave Blumenthal

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program in a file named "GNU General Public License".
If not, write to the Free Software Foundation, 675 Mass Ave,
Cambridge, MA 02139, USA.

\**********************************************************************/

#include "msg timing.h"

#define BlockSize 10
#define CorrectTime 3

void TwoCorner(GrafPtr, GrafPtr, int, int);

/* One region starts at the top of the screen and moves down the right side
   (making triangles with the topleft of the screen).  The other region starts
   at the bottom of the screen and moves up the left side (making triangles with
   the bottomright of the screen). */
   
void TwoCorner(GrafPtr sourceGrafPtr, GrafPtr destGrafPtr, int theWindowHeight, int theWindowWidth)
{
	RgnHandle	curregion;
	Rect		source;
	int			gap;
	
	curregion=NewRgn();
	source.top=source.left=0;
	source.bottom=theWindowHeight;
	source.right=theWindowWidth;
	
	gap=0;
	do
	{
		StartTiming();
		SetEmptyRgn(curregion);
		MoveTo(0,0);
		OpenRgn();
			Line(theWindowWidth,gap);
			Line(0,BlockSize);
			LineTo(0,0);
			MoveTo(theWindowWidth,theWindowHeight);   /* region is discontinuous */
			Line(-theWindowWidth,-gap);                  /* but this is much faster */
			Line(0,-BlockSize);                             /* than two regions & two  */
			LineTo(theWindowWidth,theWindowHeight);   /* CopyBits */
		CloseRgn(curregion);
		CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
			&source, &source, 0, curregion);
		gap+=BlockSize;
		TimeCorrection(CorrectTime);
	}
	while (gap<theWindowHeight+BlockSize);
	
	CopyBits(&(sourceGrafPtr->portBits), &(destGrafPtr->portBits),
		&source, &source, 0, 0L);   /* in case we missed any bits */
	
	DisposeRgn(curregion);
}
